define(["esri/widgets/Search"], function(Search) {
    return function(pointsLayer, mapView, pointSelHandler) {
        const widget = new Search({
            view: mapView,
            sources: [{
                featureLayer: pointsLayer,
                searchFields: ["title", "longtext"],
                displayField: "title",
                exactMatch: false,
                outFields: ["objectid"],
                placeholder: "Search for a memory"
            }],
            popupEnabled: false,
            resultGraphicEnabled: false
        });

        widget.watch("results", function(results) {
            if(results != null && results.length > 0 && results[0].results != null && results[0].results.length > 0) {
                const objId = results[0].results[0].feature.attributes.objectid;

                // Wait for mapView to be ready
                mapView.whenLayerView(pointsLayer).then(function(layerView) {
                    // Look for matching point in all of the features
                    layerView.queryFeatures().then(function(queryResults) {
                        const point = queryResults.find(function(result) {
                            return result.attributes.objectid == objId;
                        });

                        if(point != null) {
                            // Make the point look like how selectPoint wants it
                            const pt = {
                                graphic: point
                            }

                            pointSelHandler.selectPoint(pt);
                        }
                    });
                });
            }
        });

        return widget;
    }
});