/*
 * Handles submitting a point based in info given in the point submission form
 */

define(["esri/geometry/Point"], function (Point) {
    /**
     * Create a new point submission handler
     * 
     * @param {*} selHandler Point selection handler
     * @param {*} featureLayer Feature layer for the KTT feature service
     */
    return function (selHandler, featureLayer, oldMaps) {
        const self = this;

        // Submit the point when the submit button is clicked
        // TODO this currently handles ALL submit buttons, assuming that there is only one on the page
        document.addEventListener("submit", function(event) {
            event.preventDefault();

            self.submit(selHandler.newPointPos.longitude, selHandler.newPointPos.latitude, oldMaps.currentYear);
        });

        /**
         * Submit a point to the feature service
         * 
         * @param {*} x X coordinate of the point to submit
         * @param {*} y Y coordinate of the point to submit
         * @param {*} year The year that is currently selected via the year select widget
         */
        this.submit = function (x, y, year) {
            const form = document.getElementById("newPointForm");

            // Create a point feature to submit to the service
            const point = {
                geometry: new Point({
                    x: x,
                    y: y
                }),
                attributes: {
                    title: form.titleInput.value,
                    name: form.nameInput.value,
                    longtext: form.descInput.value,
                    date: form.dateInput.value,
                    dest: form.dateEstimated.checked ? 1 : 0, // whether the date is estimated
                    syear: year
                }
            };

            // Make the point look like what attachmentify and selectPoint expect
            const pt = {
                graphic: point
            }

            // Add the point to the feature service
            featureLayer.applyEdits({
                addFeatures: [ point ]
            }).then(function (response) {;
                pt.graphic.attributes.objectid = response.addFeatureResults[0].objectId;

                // Add attachments
                attachmentifyPoint(pt);

                for (let i = 1; i <= 3; i++) { // Go through all 3 upload files
                    const imageInput = document.getElementById("imageinput" + i);
                    if (imageInput.files.length > 0) {
                        pt.addAttachment(imageInput.files[0], function(response) {
                            selHandler.selectPoint(pt);
                        });
                    }
                }

                selHandler.selectPoint(pt);
            });
        }
    }
});