const ATTACH_PREFIX_URL = "http://gis-core.sabu.mtu.edu:6080/arcgis/rest/services/CS4760/KeTT_Story_Points/FeatureServer/0/"

/*
 * Adds attachment-related utility functions to a point
 */
function attachmentifyPoint(pt) {
    if ('attachmentified' in pt && pt.attachmentified) { // Point is already attachmentified
        return;
    }

    pt.attachmentified = true;

    pt.featureURL = ATTACH_PREFIX_URL + pt.graphic.attributes.objectid + '/';
    pt.attachURL = pt.featureURL + 'attachments';

    // Gets a list of attachments associated with the point
    pt.getAttachments = function (onRetrieve) {
        if ('attachments' in pt && pt.attachments != null) { // Already retrieved
            onRetrieve(pt.attachments);
        } else {
            pt.updateAttachments(onRetrieve);
        }
    };

    // Forces an update of the attachments, even if they were already cached
    pt.updateAttachments = function (onRetrieve) {
        const attachListURL = pt.attachURL + '?f=json';

        $.getJSON(attachListURL).then(function (resp) {
            pt.attachments = resp['attachmentInfos'];
            onRetrieve(pt.attachments);
        });
    }

    pt.addAttachment = function (file, onAdded, filename) {
        pt.featureURL = ATTACH_PREFIX_URL + pt.graphic.attributes.objectid + '/';
        pt.attachURL = pt.featureURL + 'attachments';

        var formData = new FormData();

        formData.append("f", "json");
        if(filename != null) {
            formData.append("attachment", file, filename);
        } else {
            formData.append("attachment", file);
        }

        var request = new XMLHttpRequest();

        request.onreadystatechange = function () {
            if (request.readyState == 4 && request.status == 200) {
                pt.updateAttachments(onAdded);
            }
        };

        request.open("POST", pt.featureURL + 'addAttachment');
        request.send(formData);
    };
}